@@ -1,8 +1,17 @@
 import re
 import urllib.parse
-from datetime import datetime
+from datetime import date, datetime, timedelta
 from types import TracebackType
-from typing import Dict, Iterable, List, Optional, Protocol, Type
+from typing import (
+    Callable,
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    Protocol,
+    Tuple,
+    Type,
+)
 
 import bs4
 import mechanicalsoup
@@ -33,6 +42,10 @@ PROFILE_HREF_RX = re.compile(r'.*/profiles/([^/]+)/(\d+)')
 NEWS_ITEM_LINK_RX = re.compile(r'/!(\d+)/news/(?P<news_id>\d+)$')
 REPLY_HEADER_RX = re.compile(
     r'(?P<from>.*)\xa0? replied [^0-9]*(?P<date>[0-9][0-9.:/ ]+)$')
+SPECIAL_DATES: Dict[str, Callable[[], date]] = {
+    'today': date.today,
+    'yesterday': lambda: date.today() - timedelta(days=1),
+}
 YEARLESS_DATE_RX = re.compile(
     r'^((0?[1-9])|[1-2][0-9]|3[01])\.((0?[1-9])|(1[0-2]))\.$')
 
@@ -68,7 +81,7 @@ class Connection:
         Log in to the site.
         """
         browser = mechanicalsoup.StatefulBrowser(raise_on_404=True)
-        browser.open(f'{url}/login')
+        browser.open(f'{url}/?langid={ENGLISH_LANG_ID}')
         browser.select_form('.login-form')
         browser['Login'] = username
         browser['Password'] = password
@@ -103,8 +116,9 @@ class Connection:
     ) -> None:
         self.url = url
         self.browser = browser
-        self.front_page = self._browse(f'/?langid={ENGLISH_LANG_ID}')
+        self.front_page = self._get_current_page_or_fail()
         links = self.front_page.find_all('a', href=True)
+        self._check_language(links)
         self.pupils = self._parse_pupils(links)
         self.new_message_counts = self._parse_new_message_counts(links)
         own_name_span = self.front_page.select_one('.name-container .teacher')
@@ -112,6 +126,15 @@ class Connection:
             raise Exception('Cannot find the span containing your name')
         self.own_name = own_name_span.text
 
+    def _check_language(self, links: Iterable[Tag]) -> None:
+        for a_elem in links:
+            if a_elem.get('href', '').endswith('passwd/settings'):
+                if a_elem.text == "Account settings":
+                    return
+                raise Exception(
+                    f"Invalid language: 'Account settings' is {a_elem.text!r}")
+        raise Exception("Cannot find element for checking language")
+
     def _parse_pupils(self, links: Iterable[Tag]) -> Dict[PupilId, Pupil]:
         """
         Get list of pupils.
@@ -163,7 +186,7 @@ class Connection:
                 origin=self.url,
                 pupil_id=pupil_id,
                 subject=x['Subject'],
-                timestamp=_parse_timestamp(x['TimeStamp']),
+                last_timestamp=_parse_timestamp(x['TimeStamp']),
                 folder=x['Folder'],
                 sender=Person(
                     name=x['Sender'],
@@ -182,11 +205,12 @@ class Connection:
                 f'Invalid message origin: '
                 f'{message_info.origin} (expected {self.url})')
         body = self._fetch_message_body(message_info.pupil_id, message_info.id)
+        timestamp = self._parse_sent_time(body)
         recipients = self._parse_recipients(body)
         message_content = self._parse_message_content(body)
         replies = self._parse_replies(body)
         message = Message.from_info_and_attrs(
-            message_info, recipients, message_content, replies)
+            message_info, timestamp, recipients, message_content, replies)
         return message
 
     def _fetch_message_body(
@@ -205,6 +229,15 @@ class Connection:
         replace_emoji_imgs(body)
         return body
 
+    def _parse_sent_time(self, body: Tag) -> datetime:
+        table_ths = body.select('table th')
+        sent_ths = [x for x in table_ths if x.text.startswith('Sent:')]
+        if len(sent_ths) == 1 and sent_ths[0].parent:
+            sent_td = sent_ths[0].parent.find('td')
+            if sent_td:
+                return _parse_timestamp(sent_td.text)
+        raise Exception(f'Cannot find table cell contaiting sending time')
+
     def _parse_recipients(self, body: Tag) -> List[Person]:
         recip_div = body.select_one('#recipients-cell')
         if not recip_div:
@@ -258,28 +291,44 @@ class Connection:
             for a_elem in page.find_all('a', href=True)
             if a_elem and a_elem.get('class')
         )
-        news_map = {
-            int(match.group('news_id')): a_elem.text.strip()
+        news_map: Dict[int, Tuple[str, Optional[datetime], bool]] = {
+            # news_id -> (subject, date)
+            int(match.group('news_id')): (a_elem.text.strip(), None, False)
             for (a_elem, match) in link_matches
             if match
         }
         for well in page.select('.well'):
+            date_h2 = well.find_previous('h2')
+            date = _parse_timestamp(date_h2.text) if date_h2 else None
             title_elem = well.find('h3')
             a_elem = well.find('a', href=True)
             href = a_elem.get('href', '') if a_elem else ''
             match = NEWS_ITEM_LINK_RX.match(href)
             if title_elem and match:
-                news_map[int(match.group('news_id'))] = title_elem.text.strip()
+                (subject, is_new) = self._parse_news_title(title_elem)
+                news_map[int(match.group('news_id'))] = (subject, date, is_new)
         return [
             NewsItemInfo(
                 id=NewsItemId(news_id),
                 origin=self.url,
                 pupil_id=pupil_id,
                 subject=subject,
+                timestamp=timestamp,
+                is_unread=is_new,
             )
-            for (news_id, subject) in sorted(news_map.items())
+            for (news_id, (subject, timestamp, is_new)) in sorted(
+                    news_map.items())
         ]
 
+    def _parse_news_title(self, title_elem: Tag) -> Tuple[str, bool]:
+        labels = set()
+        for label in title_elem.select("span.label"):
+            labels.add(label.text.lower())
+            label.replace_with('')
+        subject = title_elem.text.strip()
+        is_new = ('new' in labels)
+        return (subject, is_new)
+
     def fetch_news_item(self, news_item_info: NewsItemInfo) -> NewsItem:
         elem = self._fetch_news_item_body(
             news_item_info.pupil_id, news_item_info.id)
@@ -331,7 +380,8 @@ class Connection:
         if profile_link:
             return self._parse_profile_link(profile_link)
         else:
-            return Person(element.text.strip())
+            text_lines = element.text.strip().splitlines() or ['']
+            return Person(text_lines[0].strip())
 
     def _parse_profile_link(self, profile_link: Tag) -> Person:
         profile_href = profile_link.get('href', '')
@@ -380,7 +430,10 @@ class _PytzTimezone(Protocol):
 
 
 def _parse_timestamp(string: str, tz: _PytzTimezone = TZ) -> datetime:
-    if YEARLESS_DATE_RX.match(string):
+    special_date_function = SPECIAL_DATES.get(string.strip().lower())
+    if special_date_function:
+        string = str(special_date_function())
+    elif YEARLESS_DATE_RX.match(string):
         string += str(datetime.now().year)
     dt = parse_datetime(string, dayfirst=(string.count('.') >= 2))
     if dt.tzinfo:
